from . import make_nested_py_modules

EXAMPLE_1 = {
    1: {
        'subpackage_prefixes': ['example_1_package'],
    }
}
EXAMPLE_2 = {
    1: {
        'subpackage_prefixes': ['example_2_package'],
        'subpackage_suffixes': [''],
        'module_prefixes': ['module_'],
        'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
    },
    2: {
        'subpackage_prefixes': ['subpackage_'],
        'subpackage_suffixes': ['a', 'b', 'c'],
        'module_prefixes': [''],
        'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
    },
}

EXAMPLE_3 = {
    1: {
        'subpackage_prefixes': ['example_3_package_'],
        'subpackage_suffixes': ['a', 'b', 'c'],
        'module_prefixes': ['module_'],
        'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
    },
    2: {
        'subpackage_prefixes': ['subpackage_'],
        'subpackage_suffixes': ['a', 'b', 'c'],
        'module_prefixes': [''],
        'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
    },
    3: {
        'subpackage_prefixes': ['subpackage_'],
        'subpackage_suffixes': ['a', 'b', 'c'],
        'module_prefixes': [''],
        'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
    },
    4: {
        'subpackage_prefixes': ['subpackage_'],
        'subpackage_suffixes': ['a', 'b', 'c'],
        'module_prefixes': [''],
        'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
    },
}

def main():
    make_nested_py_modules.main(package_levels=EXAMPLE_1, start=1, end=7, directory_removal_prefixes=['example_1_package'])
    make_nested_py_modules.main(package_levels=EXAMPLE_2, start=1, end=2, directory_removal_prefixes=['example_2_package'])
    make_nested_py_modules.main(package_levels=EXAMPLE_3, start=1, end=5, directory_removal_prefixes=['example_3_package'])


if __name__ == '__main__':
    main()



import collections.abc
import itertools
import os
import shutil


def main(package_levels=None, start=1, end=2, directory_removal_prefixes=None):
    none_type = type(None)
    validate_instance_is_from_class(package_levels, (none_type, collections.abc.Mapping), 'package_levels')
    validate_instance_is_from_class(start, int, 'start')
    validate_instance_is_from_class(end,   int, 'end')
    validate_instance_is_from_class(directory_removal_prefixes, (none_type, collections.abc.Sequence), 'directory_removal_prefixes')
    if directory_removal_prefixes is None: directory_removal_prefixes = ['package_']
    remove_script_created_contents(directory_removal_prefixes)
    if package_levels is None:
        package_levels = {
            1: {
                'subpackage_prefixes': ['package_'],
                'subpackage_suffixes': ['a', 'b', 'c'],
                'module_prefixes': ['module_'],
                'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
            },
            2: {
                'subpackage_prefixes': ['subpackage_'],
                'subpackage_suffixes': ['a', 'b', 'c'],
                'module_prefixes': [''],
                'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
            },
            3: {
                'subpackage_prefixes': ['subpackage_'],
                'subpackage_suffixes': ['a', 'b', 'c'],
                'module_prefixes': [''],
                'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
            },
            4: {
                'subpackage_prefixes': ['subpackage_'],
                'subpackage_suffixes': ['a', 'b', 'c'],
                'module_prefixes': [''],
                'module_suffixes': ['a', 'b', 'c', 'd', 'e'],
            },
        }
    create_packages(package_levels, start, end, base_path='.', base_name='', __init__files=[])


def validate_instance_is_from_class(obj, acceptable_types, variable_name):
    if isinstance(acceptable_types, tuple): formatted_acceptable_types = f'one of the following types: {acceptable_types}' # multiple acceptable types
    else:                                   formatted_acceptable_types = f'an instance of {acceptable_types}'              # one      acceptable type
    variable_reference = f'variable `{variable_name}`'
    error_message = f'''
    Invalid value provided for {variable_reference}
      You provided {obj} (type {type(obj)}) for {variable_reference}, but
      {variable_reference} needs to be {formatted_acceptable_types}
    '''
    if not isinstance(obj, acceptable_types):
        raise TypeError(error_message)


def remove_script_created_contents(directory_removal_prefixes):
    top_level_path, top_level_nested_directories, top_level_files = next(os.walk('.'))
    # print(top_level_path, top_level_nested_directories, top_level_files)
    for nested_directory in top_level_nested_directories:
        if any(nested_directory.startswith(prefix) for prefix in directory_removal_prefixes):
            remove_directory(nested_directory)


def remove_directory(directory):
    shutil.rmtree(directory)


def create_packages(package_levels, start, end, base_path, base_name, __init__files):
    if start <= end:
        current_level     = start
        base_name         = base_name + '.' if base_name else ''
        current_package   = package_levels.get(current_level, {})
        current_subpackage_prefixes = current_package.get('subpackage_prefixes', [f'level_{current_level}_'])
        current_subpackage_suffixes = current_package.get('subpackage_suffixes', ['_package'])
        validate_instance_is_from_class(current_subpackage_prefixes, collections.abc.Sequence, f'''package_levels[{current_level}]['subpackage_prefixes']''')
        validate_instance_is_from_class(current_subpackage_suffixes, collections.abc.Sequence, f'''package_levels[{current_level}]['subpackage_suffixes']''')
        for subpackage_prefix, subpackage_suffix in itertools.product(current_subpackage_prefixes, current_subpackage_suffixes):
            package_name, package_path      = determine_subpackage_info(subpackage_prefix, subpackage_suffix, base_path, base_name)
            module_prefixes , module_suffixes = determine_module_info_for_subpackage(current_package)
            validate_instance_is_from_class(module_prefixes, collections.abc.Sequence, f'''package_levels[{current_level}]['module_prefixes']''')
            validate_instance_is_from_class(module_suffixes, collections.abc.Sequence, f'''package_levels[{current_level}]['module_suffixes']''')
            create_subpackage(package_path)
            for __init__file, __init__package_path in __init__files:
                relative_subpackage_path, relative_subpackage_name = determine_relative_subpackage_info(package_name, __init__package_path)
                __init__file.write(f'from {relative_subpackage_path} import {relative_subpackage_name}\n')
            subpackage__init__filepath = os.path.join(package_path, '__init__.py')
            with open(subpackage__init__filepath, mode='w', encoding='utf-8', buffering=-1) as subpackage__init__file:
                create_modules_for_subpackage(package_path, package_name, module_suffixes, module_prefixes, subpackage__init__file)
                __init__files.append((subpackage__init__file, package_name))
                create_packages(package_levels, start+1, end, base_path=package_path, base_name=package_name, __init__files=__init__files)
                __init__files.pop()


def determine_subpackage_info(subpackage_prefix, subpackage_suffix, base_path, base_name):
    subpackage_name      = subpackage_prefix + subpackage_suffix
    full_subpackage_name = base_name + subpackage_name
    full_subpackage_path = os.path.join(base_path, subpackage_name)
    print(f'package_name is {full_subpackage_name}')
    return full_subpackage_name, full_subpackage_path


def determine_module_info_for_subpackage(current_package):
    module_suffixes = current_package.get('module_suffixes', ['_name'])
    module_prefixes = current_package.get('module_prefixes', ['module_'])
    return module_prefixes, module_suffixes


def determine_relative_subpackage_info(package_name, __init__package_path):
    # imports still work with the hard coded path but provide less flexibility
    # than relative imports
    #     renaming a base package would require renaming the
    #     subpackage `import` statement in the `__init__` module when
    #     the import is a hard coded import
    #         for example, renaming `package_name` to `new_package_name` would require changing
    #         `import package_name.to.subpackage.module`
    #         to
    #         `import new_package_name.to.subpackage.module`
    #         in the package_name.to.__init__.py module (new_package_name.to.__init__ module after the rename)
    #     renaming a base package would NOT require renaming the
    #     subpackage `import` statement in the `__init__` module when
    #     the import is a relative import
    #         for example, renaming `package_name` to `new_package_name` would require no change to
    #         `from . import subpackage.module`
    #         in the package_name.to.__init__.py module (new_package_name.to.__init__ module after the rename)
    relative_subpackage_path_start = len(__init__package_path)
    normalize_subpackage_path      = package_name[relative_subpackage_path_start:]
    subpackages                    = normalize_subpackage_path.split('.')
    if len(subpackages) == 2:
        relative_subpackage_path = '.'
        relative_subpackage_name = subpackages[1]
    else:
        relative_subpackage_path = '.'.join(subpackages[:-1])
        relative_subpackage_name = subpackages[-1]
    return relative_subpackage_path, relative_subpackage_name


def create_subpackage(package_path, mode=511, exist_ok=False):
    os.makedirs(name=package_path, mode=mode, exist_ok=exist_ok)


def create_modules_for_subpackage(package_path, package_name, module_suffixes, module_prefixes, subpackage__init__file):
    for module_prefix, module_suffix in itertools.product(module_prefixes, module_suffixes):
        module_name = f'{module_prefix}{module_suffix}'
        subpackage__init__file.write(f'from . import {module_name}\n')
        module_file_path = os.path.join(package_path, f'{module_name}.py')
        with open(module_file_path, mode='w', encoding='utf-8', buffering=-1) as file:
            file.write(f'''print('{package_name}.{module_name}')\n''')
            file.write('')


if __name__ == '__main__':
    main()



print('Entered test_project_python.__main__ :)')

def function_in__main__():
    print('Ran test_project_python.__main__.function_in__main__')
    print('''
    To better understand how commands from the 'console_scripts' key inside
    the entry_points dictionary in setup.py works, run and compare
        command_name
        command_name_for_function_in__main__
        python3   -m test_project_python        # Linux/macOS
        python    -m test_project_python        # Windows
        custom_py -m test_project_python        # if you use a custom alias for python command
    This assumes you've already installed the package with:
        pip install test_project_python         # installs from PyPI
        pip install .                           # installs local version (you need to clone this repo and then run this from the test_project/python directory)
    ''')

if __name__ == '__main__':
    print('__name__ == "__main__" is True for test_project_python.__main__')
    function_in__main__()



# https://pypi.org/classifiers/
# https://test.pypi.org/pypi?%3Aaction=list_classifiers
# https://github.com/pypa/sampleproject/blob/master/setup.py
# https://packaging.python.org/guides/distributing-packages-using-setuptools/
from setuptools import setup, find_packages


with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()


setup(
    name                          = 'test-project-python',
    version                       = '0.0.2',
    description                   = 'Test project to see how packaging works on PyPI.',
    long_description              = long_description,
    long_description_content_type = 'text/markdown',
    url                           = 'https://github.com/shailshouryya/test-project',
    author                        = 'shailshouryya',
    author_email                  = 'slowbutsteady1234+github@gmail.com',
    license                       = 'MIT',


    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'License :: Free For Educational Use',
        'License :: Free For Home Use',
        'License :: Free for non-commercial use',
        'License :: Freely Distributable',
        'License :: Freeware',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Microsoft :: Windows :: Windows 10',
        'Operating System :: Microsoft :: Windows :: Windows 11',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Home Automation',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Version Control :: Git',
        'Topic :: System :: Installation/Setup',
        'Topic :: Text Processing :: Markup :: HTML'
    ],
    keywords='PyPI packaging package subpackage module namespace plugin extensions import macos windows linux setuptools setup',


    # http://code.nabla.net/doc/setuptools/api/setuptools/setuptools.find_packages.html
    # https://stackoverflow.com/questions/51286928/what-is-where-argument-for-in-setuptools-find-packages
    packages=find_packages(), # reawd links above to understand how to use variations below
    # packages=find_packages(exclude=['includes_all_directories', 'other_than', 'the_ones_here']),
    # packages=find_namespace_packages(include=['only_includes_this']),
    # package_dir={'':'src'},


    python_requires  = '>=3.0.*, <4',
    install_requires = [],  # Optional
    # https://packaging.python.org/discussions/install-requires-vs-requirements/


    # If there are data files included in your packages that need to be installed, specify them here.
    # If using Python 2.6 or earlier, then these have to be included in MANIFEST.in as well.
    package_data = {  # Optional
        # 'sample': ['package_data.dat'],
    },
    # Although 'package_data' is the preferred approach, in some case you may need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],  # Optional


    # # To provide executable scripts, use entry points in preference to the "scripts" keyword.
    # # Entry points provide cross-platform support and allow `pip` to create the appropriate form of executable for the target platform.
    # # For example, the following would provide a command called `test_project_python` which
    # #   tries to run a __main__ function (or whatever comes after the ":" (colon) for the defined command) from
    # #   the test_project_python package when invoked directly from the command line
    # # This means we
    # #   - load test_project_python.__init__
    # #     - the __init__ module subsequently loads all other modules in the test_project_python package including the __main__ module
    # #   - try to call a __main__ function from whatever is loaded in this (__init__ module) namespace
    # #   - fail since the __init__ module loads the __main__ module
    # #     - therefore, there is no __main__ function in the test_project_python namespace since
    # #       the __main__ we loaded is a reference to the test_project_python.__main__ module
    # # - see https://stackoverflow.com/a/782984 for a better explanation
    # entry_points={  # Optional
    #    'console_scripts': [
    #        'test_project_python=test_project_python:__main__',
    #    ],
    # },
    #
    #
    # # contents of /home/USERNAME/.local/bin/test_project_python after running the above:
    #
    # # #!/path/to/your/bin/for/python_command
    # # # -*- coding: utf-8 -*-
    # # import re
    # # import sys
    # # from test_project_python import __main__
    # # if __name__ == '__main__':
    # #     sys.argv[0] = re.sub(r'(-script\.pyw|\.exe)?$', '', sys.argv[0])
    # #     sys.exit(__main__())
    #
    # # the test_project_python command FAILS
    # # if a `__main__` function is not defined inside
    # # test_project_python (inside the test_project_python.__init__ namespace):
    #
    # # $ test_project_python
    # # Entered test_project_python.__main__ :)
    # # Traceback (most recent call last):
    # #   File "/home/USERNAME/.local/bin/test_project_python", line 8, in <module>
    # #     sys.exit(__main__())
    # # TypeError: 'module' object is not callable
    # # potential useful references:
    # #   https://stackoverflow.com/questions/4534438/typeerror-module-object-is-not-callable
    # #   https://stackoverflow.com/questions/5280203/what-does-this-mean-exit-main

     entry_points={  # Optional
       'console_scripts': [
           'command_name=test_project_python:function_name',                                        # compare the output of `command_name`                         versus `python3 -m test_project_python`
           'command_name_for_function_in__main__=test_project_python.__main__:function_in__main__', # compare the output of `command_name_for_function_in__main__` versus `python3 -m test_project_python`
       ],
    },

    project_urls = {
        'Bug Reports':  'https://github.com/shailshouryya/test-project/issues',
        'PyPi Funding': 'https://donate.pypi.org',
        'Source':       'https://github.com/shailshouryya/test-project'
    },
)

'''
files created by setuptools for the commands specified in entry_points['console_scripts'] for reference:


# contents of /path/to/your/bin/for/python_command/command_name:
#!/path/to/your/bin/for/python_command
# -*- coding: utf-8 -*-
import re
import sys
from test_project_python import function_name
if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw|\.exe)?$', '', sys.argv[0])
    sys.exit(function_name())


# contents of /path/to/your/bin/for/python_command/command_name_for_function_in__main__:
#!/path/to/your/bin/for/python_command
# -*- coding: utf-8 -*-
import re
import sys
from test_project_python.__main__ import function_in__main__
if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw|\.exe)?$', '', sys.argv[0])
    sys.exit(function_in__main__())

'''



__version__ = '0.0.2'

print('Entered test_project_python.__init__ :)')

def function_name():
    print('Ran test_project_python.__init__.function_name')
    return True


if __name__ == '__main__':
    print('__name__ == "__main__" is True for test_project_python.__init__')
    function_name()
